import { Draft } from './immupdate';
import { pipe, Wrapper, Lifted } from './lift';
import { MapWrapper } from './map';
import { SetWrapper } from './set';
/** An Array wrapper providing extra functionalities and more chaining opportunities */
export declare class ArrayWrapper<T> {
    private _value;
    constructor(_value: ReadonlyArray<T>);
    private _isLiftWrapper;
    value(): readonly T[];
    clone(): ArrayWrapper<T>;
    private _clone;
    /**
     * Appends one item at the end of the Array.
     */
    append(item: T): ArrayWrapper<T>;
    /**
     * Appends an Array of items at the end of the Array.
     */
    appendAll(items: T[]): ArrayWrapper<T>;
    /**
     * Filters all the falsy elements out of this Array.
     * All occurences of false, null, undefined, 0, "" will be removed.
     */
    compact<A>(this: ArrayWrapper<A | false | null | undefined | 0 | ''>): ArrayWrapper<A>;
    /**
     * Counts the items satisfying a predicate.
     */
    count(predicate: (item: T, index: number) => boolean): number;
    /**
     * Maps this Array's items, unless void or undefined is returned, in which case the item is filtered.
     */
    collect<B>(iterator: (item: T, index: number) => B | undefined | void): ArrayWrapper<B>;
    /**
     * Creates an array without any duplicate item.
     * If a key function is passed, items will be compared based on the result of that function;
     * if not, they will be compared using strict equality.
     */
    distinct(getKey?: (item: T, index: number) => string | number): ArrayWrapper<T>;
    /**
     * Drops the first 'count' items from this Array.
     */
    drop(count: number): ArrayWrapper<T>;
    /**
     * Drops the last 'count' items from this Array.
     */
    dropRight(count: number): ArrayWrapper<T>;
    /**
     * Filters this array by aplying a predicate to all items and refine its type.
     */
    filter<A extends T>(predicate: (item: T, index: number) => item is A): ArrayWrapper<A>;
    /**
     * Filters this array by aplying a predicate.
     */
    filter(predicate: (item: T, index: number) => boolean): ArrayWrapper<T>;
    /**
     * Returns the first element in this Array or undefined.
     */
    first(): T | undefined;
    /**
     * Maps this Array to an Array of Array | ArrayWrapper using a mapper function then flattens it.
     */
    flatMap<B>(fun: (item: T, index: number) => B[]): ArrayWrapper<B>;
    /**
     * Maps this Array to an Array of Array | ArrayWrapper using a mapper function then flattens it.
     */
    flatMap<B>(fun: (item: T, index: number) => ArrayWrapper<B>): ArrayWrapper<B>;
    /**
     * Flattens this Array of Arrays.
     */
    flatten<A>(this: ArrayWrapper<ReadonlyArray<A>>): ArrayWrapper<A>;
    flatten<A>(this: ArrayWrapper<A[]>): ArrayWrapper<A>;
    /**
     * Folds this Array into a single value, using a starting value.
     */
    fold<V>(startValue: V, func: (acc: V, value: T, index: number) => V): Lifted<V>;
    /**
     * Returns the item found at the provided index or undefined.
     */
    get(index: number): T | undefined;
    /**
     * Creates a Map where keys are the results of running each element through a discriminator function.
     * The corresponding value of each key is an array of the elements responsible for generating the key.
     */
    groupBy<K extends string | number>(discriminator: (item: T, index: number) => K): MapWrapper<K, ReadonlyArray<T>>;
    /**
     * Inserts an item at a specified index.
     */
    insert(index: number, item: T): ArrayWrapper<T>;
    /**
     * Returns the item found at the last index or undefined.
     */
    last(): T | undefined;
    /**
     * Maps this Array using a mapper function.
     */
    map<B>(fun: (item: T, index: number) => B): ArrayWrapper<B>;
    /**
     * Removes the item found at the specified index.
     */
    removeAt(index: number): ArrayWrapper<T>;
    /**
     * Reverses the Array.
     */
    reverse(): ArrayWrapper<T>;
    /**
     * Sorts the Array in ascending order, using one or more iterators specifying which field to compare.
     * For strings, localCompare is used.
     * The sort is stable if the browser uses a stable sort (all modern engines do)
     */
    sort(...fields: Array<SortOnField<T>>): ArrayWrapper<T>;
    /**
     * Takes the first 'count' items from this Array.
     */
    take(count: number): ArrayWrapper<T>;
    /**
     * Takes the last 'count' items from this Array.
     */
    takeRight(count: number): ArrayWrapper<T>;
    /**
     * Converts this Array to a Set.
     */
    toSet(): SetWrapper<T>;
    /**
     * Make mutable modifications to a draft then return a new Array.
     * Example: lift([{a: 1}]).update(draft => {draft[0]!.a = 10})
     */
    update(updateFunction: (draft: Draft<ReadonlyArray<T>>) => void): ArrayWrapper<T>;
    /**
     * Updates an item at the specified index.
     */
    updateAt(index: number, updater: (item: T) => Wrapper<T>): ArrayWrapper<T>;
    /**
     * Updates an item at the specified index.
     */
    updateAt(index: number, updater: (item: T) => T): ArrayWrapper<T>;
    /**
     * Pipes this Array with an arbitrary transformation function.
     */
    pipe: typeof pipe;
}
export declare function range(start: number, stop?: number, step?: number): ArrayWrapper<number>;
declare type SortOnField<T> = ((field: T) => string | null | undefined) | ((field: T) => number | null | undefined);
export {};
